'TinyMCE Translation Checker v1.0, by Liron Newman (http://eesh.net/tmtc)
'
'Allows translators to easily check their if their translations are up to date
'with changes in new versions.

Dim Engine, AllOutput, AppHeader
Engine = UCase(Right(WScript.FullName,11))
AppHeader = "TinyMCE Translation Checker v1.0, by Liron Newman (http://eesh.net/tmtc)"

If WScript.Arguments.Count < 2 Then
	HelpStr =  _
	        AppHeader & vbNewLine & vbNewLine & _
		"Error: Not enough parameters." & vbNewLine & vbNewLine & _
		"Syntax: [CScript.exe] tmtc.vbs <BasePath> <LangCode> [MatchAgainst]" & vbNewLine & vbNewLine & _
		"Where:" & vbNewLine & _
		"BasePath	- The path all language files are stored under, usually the TinyMCE folder" & vbNewLine & _
		"LangCode	- The language code for your translation, like en, it, hu, he, ja_utf-8..." & vbNewLine & _
		"MatchAgainst	- Optional: The language to match against. Defaults to English (en) for obvious reasons"
	If Engine = "WSCRIPT.EXE" Then
		MsgBox HelpStr, 48, "TinyMCE Translation Checker"
	Else
		WScript.Echo HelpStr
	End If

	WScript.Quit
End If

Dim fso, BasePath, LangCode, MatchAgainst, BaseFolder
Set fso = CreateObject("Scripting.FileSystemObject")

BasePath = WScript.Arguments(0)
LangCode = WScript.Arguments(1)

If WScript.Arguments.Count > 2 Then
	MatchAgainst = WScript.Arguments(2)
Else
	MatchAgainst = "en" 'The default
End If

Set BaseFolder = fso.GetFolder(BasePath)

MyEcho AppHeader & vbNewLine & vbNewLine

MyEcho "Now checking language files. If you don't see any comments under here, this is a good thing!" & vbNewLine

RecurseCompare BaseFolder

MyEcho "Done."

If Engine = "WSCRIPT.EXE" Then
	'Can't use MsgBox - It truncates long output
	'MsgBox AllOutput & vbNewLine & vbNewLine & "To save this output, try running this script with CScript.exe under the command console, where you can copy it to the clipboard easily.", 64, "TinyMCE Translation Checker - Output"
	
	Dim tfolder, fil, nam
	Const TemporaryFolder = 2
	Set tfolder = fso.GetSpecialFolder(TemporaryFolder)
	Set fil = tfolder.CreateTextFile("tmtc-output.txt")
	fil.WriteLine "*** TinyMCE Translation Checker - Output:" & vbNewLine & vbNewLine 
	fil.Write AllOutput
	fil.Close
	
	Set WshShell = WScript.CreateObject("WScript.Shell")
	WshShell.Run "%windir%\notepad " & tfolder.Path & "\tmtc-output.txt"
End If

Sub RecurseCompare (fld) 'Gets a Folder object
	Dim SrcFile, TestedFile, subfld

	SrcFile=fld.Path & "\" & MatchAgainst & ".js"
	TestedFile=fld.Path & "\" & LangCode & ".js"
	
	If fso.FileExists(SrcFile) Then 'We care about this folder
		If fso.FileExists(TestedFile) Then 'We need to compare string names
			MatchStringNames SrcFile, TestedFile
		Else 'Missing a file!
			MyEcho "* ERROR:   " & TestedFile & " is missing!"
		End If
	Else
		If fso.FileExists(TestedFile) Then 'Just a translation, but no source?
			MyEcho "* WARNING: " & TestedFile & " has no matching " & MatchAgainst & ".js. Check if it is necessary for this version of TinyMCE."
		End If
	End If
	
	For Each subfld in fld.SubFolders 'Now recurse subfolders
		RecurseCompare subfld
	Next
	
End Sub

Sub MatchStringNames (SrcName, TestedName)
	Dim src, test, srcall, testall, tmpstr
	Const ForReading = 1 'Sometime VBScript "forgets" this...

	Set src = fso.OpenTextFile(SrcName, ForReading)
	Set test = fso.OpenTextFile(TestedName, ForReading)
	
	srcall = src.ReadAll
	testall = test.ReadAll
	
	src.Close
	test.Close

	Dim re, matches
	Set re = New RegExp

	re.Global = True
	re.IgnoreCase = False 'JavaScript - Case matters
	re.Pattern = "tinyMCE\.addToLang\s*\(\s*'(.*)'\s*,\s*\{" 'Find addToLang value

	Set matches = re.Execute(srcall)
	tmpstr = matches(0).SubMatches(0)

	Set matches = re.Execute(testall)
	If Not (tmpstr = matches(0).SubMatches(0)) Then
		MyEcho "* ERROR:   " & TestedName & " addToLang value doesn't match " & MatchAgainst & ".js"
	End If

	re.Pattern = "\n\s*(\w+)\s*:" 'Find string names

	Set matches = re.Execute(srcall)
	For Each match in matches
		re.Pattern = "\n\s*" & match.SubMatches(0) & "\s*:" 'Find this string name
		If Not re.Test(testall) Then
			MyEcho "* ERROR:   " & TestedName & " misses the '" & match.SubMatches(0) & "' string found in " & MatchAgainst & ".js"
		End If
	Next

	'Find if any of the strings misses a comma in the end (Except the last one)
	re.Pattern = "\n\s*(\w+)\s*:.*[\)'""]\s*[^,][\t ]*\n\s*\w+\s*:"
  
	Set matches = re.Execute(testall)
	For Each match in matches
		MyEcho "* ERROR:   " & TestedName & " misses a comma after the '" & match.SubMatches(0) & "' string (At the end of the line). This will cause Javascript errors and will not work."
	Next


	'Find if the last string has a comma at the end (It shouldn't)
	re.Pattern = ",\s*\}\s*\)\s*;\s*$"
	If re.Test(testall) Then
		MyEcho "* ERROR:   " & TestedName & " has a comma after the last string. This will cause Javascript errors and will not work."
	End If
End Sub

Sub MyEcho (str)
	If Engine = "CSCRIPT.EXE" Then
		WScript.Echo str
	Else
		AllOutput = AllOutput & str & vbNewLine
	End If
End Sub	
